/* arch/arm/mach-s3c2410/include/mach/regs-s3c2416.h
 *
 * Copyright (c) 2009 Yauhen Kharuzhy <jekhor@gmail.com>,
 *	as part of OpenInkpot project
 * Copyright (c) 2009 Promwad Innovation Company
 *	Yauhen Kharuzhy <yauhen.kharuzhy@promwad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * S3C2416 specific register definitions
*/

#ifndef __ASM_ARCH_REGS_S3C2416_H
#define __ASM_ARCH_REGS_S3C2416_H "s3c2416"

#define S3C2416_SWRST		(S3C24XX_VA_CLKPWR + 0x44)
#define S3C2416_SWRST_RESET	(0x533C2416)

/* see regs-power.h for the other registers in the power block. */

#endif	/* __ASM_ARCH_REGS_S3C2416_H */

